@@ -1,5 +1,5 @@
 /* xgettext sh backend.
-   Copyright (C) 2003, 2005-2009, 2015-2016 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2009, 2014, 2018-2019 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -31,7 +31,14 @@
 
 #include "message.h"
 #include "xgettext.h"
+#include "xg-pos.h"
+#include "xg-mixed-string.h"
+#include "xg-arglist-context.h"
+#include "xg-arglist-callshape.h"
+#include "xg-arglist-parser.h"
+#include "xg-message.h"
 #include "error.h"
+#include "error-progname.h"
 #include "xalloc.h"
 #include "hash.h"
 #include "gettext.h"
@@ -112,8 +119,12 @@ init_keywords ()
          xgettext.texi!  */
       x_sh_keyword ("gettext");
       x_sh_keyword ("ngettext:1,2");
+      /* Note: There is also special handling for 'gettext' and 'ngettext'
+         in read_command, below.  */
       x_sh_keyword ("eval_gettext");
       x_sh_keyword ("eval_ngettext:1,2");
+      x_sh_keyword ("eval_pgettext:1c,2");
+      x_sh_keyword ("eval_npgettext:1c,2,3");
       default_keywords = false;
     }
 }
@@ -127,18 +138,14 @@ init_flag_table_sh ()
   xgettext_record_flag ("eval_gettext:1:sh-format");
   xgettext_record_flag ("eval_ngettext:1:sh-format");
   xgettext_record_flag ("eval_ngettext:2:sh-format");
+  xgettext_record_flag ("eval_pgettext:2:sh-format");
+  xgettext_record_flag ("eval_npgettext:2:sh-format");
+  xgettext_record_flag ("eval_npgettext:3:sh-format");
 }
 
 
 /* ======================== Reading of characters.  ======================== */
 
-/* Real filename, used in error messages about the input file.  */
-static const char *real_file_name;
-
-/* Logical filename and line number, used to label the extracted messages.  */
-static char *logical_file_name;
-static int line_number;
-
 /* The input file stream.  */
 static FILE *fp;
 
@@ -152,8 +159,8 @@ do_getc ()
   if (c == EOF)
     {
       if (ferror (fp))
-        error (EXIT_FAILURE, errno, _("\
-error while reading \"%s\""), real_file_name);
+        error (EXIT_FAILURE, errno,
+               _("error while reading \"%s\""), real_file_name);
     }
   else if (c == '\n')
    line_number++;
@@ -468,6 +475,24 @@ string_of_word (const struct word *wp)
   return str;
 }
 
+/* Convert a t_string token to a char*, ignoring the first OFFSET bytes.  */
+static char *
+substring_of_word (const struct word *wp, size_t offset)
+{
+  char *str;
+  int n;
+
+  if (!(wp->type == t_string))
+    abort ();
+  n = wp->token->charcount;
+  if (!(offset <= n))
+    abort ();
+  str = XNMALLOC (n - offset + 1, char);
+  memcpy (str, wp->token->chars + offset, n - offset);
+  str[n - offset] = '\0';
+  return str;
+}
+
 
 /* Whitespace recognition.  */
 
@@ -1068,7 +1093,8 @@ read_word (struct word *wp, int looking_for, flag_context_ty context)
                       string.chars[string.charcount++] = (unsigned char) c;
                     }
                   remember_a_message (mlp, NULL, string_of_token (&string),
-                                      context, &pos, NULL, savable_comment);
+                                      false, context, &pos, NULL,
+                                      savable_comment, false);
                   free_token (&string);
 
                   error_with_progname = false;
@@ -1233,8 +1259,9 @@ read_command (int looking_for, flag_context_ty outer_context)
 
               pos.file_name = logical_file_name;
               pos.line_number = inner.line_number_at_start;
-              remember_a_message (mlp, NULL, string_of_word (&inner),
-                                  inner_context, &pos, NULL, savable_comment);
+              remember_a_message (mlp, NULL, string_of_word (&inner), false,
+                                  inner_context, &pos, NULL, savable_comment,
+                                  false);
             }
         }
 
@@ -1250,6 +1277,8 @@ read_command (int looking_for, flag_context_ty outer_context)
         }
       else
         {
+          bool matters_for_argparser = true;
+
           if (argparser == NULL)
             {
               /* This is the function position.  */
@@ -1282,24 +1311,82 @@ read_command (int looking_for, flag_context_ty outer_context)
             {
               /* These are the argument positions.  */
               if (inner.type == t_string)
-                arglist_parser_remember (argparser, arg,
-                                         string_of_word (&inner),
-                                         inner_context,
-                                         logical_file_name,
-                                         inner.line_number_at_start,
-                                         savable_comment);
-
-              if (arglist_parser_decidedp (argparser, arg))
                 {
-                  /* Stop looking for arguments of the last function_name.  */
-                  /* FIXME: What about context_iter?  */
-                  arglist_parser_done (argparser, arg);
-                  shapes = NULL;
-                  argparser = NULL;
+                  bool accepts_context =
+                    ((argparser->keyword_len == 7
+                      && memcmp (argparser->keyword, "gettext", 7) == 0)
+                     || (argparser->keyword_len == 8
+                         && memcmp (argparser->keyword, "ngettext", 8) == 0));
+                  if (accepts_context && argparser->next_is_msgctxt)
+                    {
+                      char *s = string_of_word (&inner);
+                      mixed_string_ty *ms =
+                        mixed_string_alloc_simple (s, lc_string,
+                                                   logical_file_name,
+                                                   inner.line_number_at_start);
+                      free (s);
+                      argparser->next_is_msgctxt = false;
+                      arglist_parser_remember_msgctxt (argparser, ms,
+                                                       inner_context,
+                                                       logical_file_name,
+                                                       inner.line_number_at_start);
+                      matters_for_argparser = false;
+                    }
+                  else if (accepts_context
+                           && ((inner.token->charcount == 2
+                                && memcmp (inner.token->chars, "-c", 2) == 0)
+                               || (inner.token->charcount == 9
+                                   && memcmp (inner.token->chars, "--context", 9) == 0)))
+                    {
+                      argparser->next_is_msgctxt = true;
+                      matters_for_argparser = false;
+                    }
+                  else if (accepts_context
+                           && (inner.token->charcount >= 10
+                               && memcmp (inner.token->chars, "--context=", 10) == 0))
+                    {
+                      char *s = substring_of_word (&inner, 10);
+                      mixed_string_ty *ms =
+                        mixed_string_alloc_simple (s, lc_string,
+                                                   logical_file_name,
+                                                   inner.line_number_at_start);
+                      free (s);
+                      argparser->next_is_msgctxt = false;
+                      arglist_parser_remember_msgctxt (argparser, ms,
+                                                       inner_context,
+                                                       logical_file_name,
+                                                       inner.line_number_at_start);
+                      matters_for_argparser = false;
+                    }
+                  else
+                    {
+                      char *s = string_of_word (&inner);
+                      mixed_string_ty *ms =
+                        mixed_string_alloc_simple (s, lc_string,
+                                                   logical_file_name,
+                                                   inner.line_number_at_start);
+                      free (s);
+                      arglist_parser_remember (argparser, arg, ms,
+                                               inner_context,
+                                               logical_file_name,
+                                               inner.line_number_at_start,
+                                               savable_comment, false);
+                    }
                 }
+
+              if (matters_for_argparser)
+                if (arglist_parser_decidedp (argparser, arg))
+                  {
+                    /* Stop looking for arguments of the last function_name.  */
+                    /* FIXME: What about context_iter?  */
+                    arglist_parser_done (argparser, arg);
+                    shapes = NULL;
+                    argparser = NULL;
+                  }
             }
 
-          arg++;
+          if (matters_for_argparser)
+            arg++;
         }
 
       free_word (&inner);
